#define PAM_SM_SESSION

#include <stdio.h>
#include <syslog.h>
#include <security/pam_modules.h>
#include <security/_pam_macros.h>
#include <security/pam_modutil.h>
#include <security/pam_ext.h>

extern int authenticate_to_special_role(const char *rolename,
					const char *password);
extern int deauthenticate_from_special_role(void);
extern int check_rbac_status(void);

PAM_EXTERN int pam_sm_open_session(pam_handle_t *pamh, int flags, int argc,
				   const char **argv)
{
	int ret;
	char *responses[2] = { NULL, NULL };
	char *username = NULL;
	char c;

	if (pam_get_item(pamh, PAM_USER, (void *)&username) != PAM_SUCCESS || username == NULL)
		return PAM_SESSION_ERR;

	ret = check_rbac_status();
	if (ret != 1)
		goto out;

	// RBAC is enabled, let's prompt for a special role
	ret = pam_prompt(pamh, PAM_PROMPT_ECHO_ON, &responses[0], "Would you like to authenticate to a special grsec role? [N] ");
	if (ret != PAM_SUCCESS) {
		_pam_overwrite(responses[0]);
		_pam_drop(responses[0]);
		return PAM_SESSION_ERR;
	}
	c = responses[0][0];
	_pam_drop(responses[0]);

	if (c != 'y' && c != 'Y')
		goto out;

	ret = pam_prompt(pamh, PAM_PROMPT_ECHO_ON, &responses[0], "Role name: ");
	if (ret != PAM_SUCCESS) {
		_pam_overwrite(responses[0]);
		_pam_drop(responses[0]);
		return PAM_SESSION_ERR;
	}
	ret = pam_prompt(pamh, PAM_PROMPT_ECHO_OFF, &responses[1], "Password: ");
	if (ret != PAM_SUCCESS) {
		_pam_overwrite(responses[1]);
		_pam_drop(responses[1]);
		return PAM_SESSION_ERR;
	}

	ret = authenticate_to_special_role(responses[0], responses[1]);
	_pam_overwrite(responses[1]);
	if (ret == 1)
		pam_syslog(pamh, LOG_ERR, "unable to authenticate to special role %s", responses[0]);
	else if (ret == 2)
		pam_syslog(pamh, LOG_NOTICE, "successfully authenticated to special role %s", responses[0]);

	_pam_overwrite(responses[0]);
	_pam_drop(responses[0]);
	_pam_drop(responses[1]);

out:
	return PAM_SUCCESS;
}


PAM_EXTERN int pam_sm_close_session(pam_handle_t *pamh, int flags, int argc,
				    const char **argv)
{
	deauthenticate_from_special_role();
	
	return PAM_SUCCESS;
}
